@@ -30,55 +30,72 @@ You use instances of :class:`MarkupBuilder` to create XML, which use the
 subsections explain how to use the builder.
 
 
-.. _ecoxipy.MarkupBuilder.item_access:
+.. _ecoxipy.MarkupBuilder.elements:
 
 Attribute and Item Retrieval - Elements
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Each failed attribute retrieval on an instance of :class:`MarkupBuilder`
-returns a method, which on calling creates an element in the output
-representation determined by the used :class:`Output` subclass instance. The
-name of the element is the same as the name of the attribute. Its children are
-the positional arguments, the attributes are determined by the named
-attributes. These `virtual` (not in the C sense) methods have the following
-signature (with ``element_name`` being substituted by the name of the element
-to create):
 
+To create elements use attribute or item retrieval on a :class:`MarkupBuilder`
+instance, this returns a method which on calling creates an element in the
+output representation determined by the used :class:`Output` subclass
+instance. The name of the element is the attribute name, or the item key
+converted to a Unicode string for item retrieval respectively. The element
+children are the positional arguments of the method call, the attributes are
+determined by the named attributes and by children of a mapping type.
 
-.. method:: ecoxipy.MarkupBuilder.method_name(self, *children, **attributes)
+These `virtual` (not in the C sense) methods have the following signature
+(with ``element_name`` being substituted by the name of the element to
+create):
+
+.. method:: .method_name(self, *children, **attributes)
 
     Creates an element in the output representation with the name
     ``method_name``.
 
     :param children:
 
-        The children (text, elements, attribute mappings and children
-        lists) of the element.
+        The children of the element.
 
     :param attributes:
 
         Each entry of this mapping defines an attribute of the created
         element, the name being the key and the value being the value
-        identified by the key.
+        identified by the key. Keys and values are converted to Unicode, byte
+        strings are decoded using the input encoding.
 
-    Those items of ``children``, which are iterable mapping types
-    (``child[name] for name in child``) define attributes. The entries of
-    ``attributes`` overwrite entries from ``children`` items.
+Each item of ``children`` is preprocessed as follows, before it is given to
+the :class:`Output` instance:
 
-    Those other items of ``children``, which are iterable or iterators
-    are replaced with their items, callables are replaced with the
-    result of their call. The same is true for all items of those, thus
-    you have to make sure no indefinite recursion occurs.
+1.  If the item is native to the output representation, it is left unchanged.
 
-    All children which are not of the output type are converted to a
-    Unicode string (byte strings are decoded with the given input
-    encoding) and create text nodes.
+2.  If the item is a Unicode or byte string, it is replace by a text node
+    created from it. Byte strings are first decoded with the given input
+    encoding.
 
+3.  If the item is of a mapping type (i.e. it has a :meth:`keys` method and
+    supports item retrieval for the keys), the entries define attributes.
+    Entries from children mappings overwrite attributes defined by mappings
+    more to the left. The entries of ``attributes`` overwrite entries from
+    ``children`` items. Keys and values are converted to Unicode values, byte
+    strings are decoded with the input encoding.
 
-You should not create elements with names starting with ``_`` in this way, as
-:class:`MarkupBuilder` instances may have such attributes, and only elements
-with names conforming to the Python identifier syntax can be created using
-attribute access. For creating such elements use the subscript syntax for
-item access, which also returns an element-building function.
+4.  If the item is iterable, it is replaced with its items after they have
+    been preprocessed.
+
+5.  If the item is a callable allowing no arguments, it is replaced with the
+    result of its call, after this has been preprocessed.
+
+6.  All other items are converted to Unicode strings and create text nodes.
+
+It is the responsibility of the caller to ensure that no infinite recursion
+occurs in steps 4 and 5.
+
+Each failed attribute retrieval on an instance of :class:`MarkupBuilder`
+returns such a method, thus you should not create elements with names starting
+with ``_`` in this way, as :class:`MarkupBuilder` instances may have such
+attributes. Of course only elements with names conforming to the Python
+identifier syntax can be created using attribute access, for creating other
+elements use the subscript syntax for item access.
 
 
 .. _ecoxipy.MarkupBuilder.slicing:
@@ -95,18 +112,20 @@ becomes a processing instruction with target ``xml-stylesheet`` and content
 ``href="style.css"``.
 
 If the slicing start argument is not specified, a function creating a XML
-document is returned. The arguments of that function become the document
-root nodes. The slicing end argument (if specified) denotes the document
-type declaration content. It can either be a 3-:func:`tuple` containing
-the document element name, the public ID and the system ID or a single
-object to be used as the document element name. If the slicing step
-argument is :const:`True`, the XML declaration is omitted and the
-document encoding is `UTF-8`. Otherwise the slicing step denotes the
-document encoding and the XML declaration is not omitted. If no step is
-specified, the encoding is `UTF-8` and the XML declaration is not
-omitted. So the slice
-``[:('html', '-//W3C//DTD XHTML 1.0 Strict//EN', 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'):True]``
-creates a XHTML 1.0 Strict document without a XML declaration.
+document is returned. The arguments of that function become the document root
+nodes and are preprocessed the same as with :ref:`element
+<ecoxipy.MarkupBuilder.elements>` children, except for attribute handling. The
+slicing end argument (if specified) denotes the document type declaration
+content. It can either be a 3-:func:`tuple` containing the document element
+name, the public ID and the system ID or a single object to be used as the
+document element name. If the slicing step argument is
+:const:`True`, the XML declaration is omitted and the document encoding is
+`UTF-8`. Otherwise the slicing step denotes the document encoding and the XML
+declaration is not omitted. If no step is specified, the encoding is `UTF-8`
+and the XML declaration is not omitted. So the slice ``[:('html', '-//W3C//DTD
+XHTML 1.0 Strict//EN',
+'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'):True]`` creates a XHTML
+1.0 Strict document without a XML declaration.
 
 
 .. _ecoxipy.MarkupBuilder.calling:
@@ -114,11 +133,11 @@ creates a XHTML 1.0 Strict document without a XML declaration.
 Calling - XML Fragments
 ^^^^^^^^^^^^^^^^^^^^^^^
 You can also call :class:`MarkupBuilder` instances to parse XML fragments
-(lists of XML Nodes). As with the ``children`` argument of the builder's
-dynamic methods, the arguments are unpacked and converted to Unicode, but
-here each of the Unicode strings is parsed as XML using
-:class:`ecoxipy.parsing.XMLFragmentParser`. This will raise a
-:class:`xml.sax.SAXException` if the XML is not well-formed.
+(lists of XML Nodes). As with the :ref:`element
+<ecoxipy.MarkupBuilder.elements>` children, the arguments preprocessed, but
+here attribute creation is not handled and each of the Unicode strings is
+parsed as XML using :class:`ecoxipy.parsing.XMLFragmentParser` (this will
+raise a :class:`xml.sax.SAXException` if the XML is not well-formed).
 
 
 .. _ecoxipy.MarkupBuilder.operators:
@@ -174,7 +193,7 @@ class MarkupBuilder(object):
     :param parser:
 
         The SAX parser to use for parsing XML. If it is :const:`None` the
-        default of :class:`ecoxipy.XMLFragmentParser` is used.
+        default of :class:`ecoxipy.parsing.XMLFragmentParser` is used.
 
     :type parser: :class:`xml.sax.xmlreader.XMLReader`
     '''
@@ -186,56 +205,71 @@ class MarkupBuilder(object):
         self._in_encoding = in_encoding
         self._parser = parser
 
+    from collections import deque as _deque
+
     def _prepare_text(self, content):
         try:
             return _unicode(content)
         except UnicodeDecodeError:
             return content.decode(self._in_encoding)
 
-    def _preprocess(self, content, target_list, target_attributes=None,
-            handle_text=None):
+    def _append_text(self, text, target_list):
+        target_list.append(self._output.text(text))
+
+    def _append_xml(self, text, target_list):
+        target_list.extend(self._parse_xml_fragment(text))
+
+    def _preprocess(self, content, target_list, target_attributes,
+            handle_text):
         if content is None:
             return
         if self._output.is_native_type(content):
             target_list.append(content)
             return
-        if handle_text is None:
-            handle_text = lambda text, target_list: target_list.append(
-                self._output.text(text))
-        if isinstance(content, bytes):
-            content = self._prepare_text(content)
-        if isinstance(content, _unicode):
+        if content.__class__ is _unicode:
             handle_text(content, target_list)
-        elif (target_attributes is not None
-                and isinstance(content, collections.Mapping)):
-            for attr_name, attr_value in content.items():
-                attr_name = self._prepare_text(attr_name)
-                if attr_value is not None:
+            return
+        if content.__class__ is bytes:
+            handle_text(self._prepare_text(content), target_list)
+            return
+        try: # mappings define attributes
+            attr_names = content.keys()
+        except AttributeError:
+            pass
+        else:
+            if target_attributes is not None:
+                for attr_name in attr_names:
+                    attr_value = content[attr_name]
                     attr_value = self._prepare_text(attr_value)
                     target_attributes[attr_name] = attr_value
-        elif isinstance(content,
-                (collections.Sequence, collections.Iterable)):
+                return
+        try: # iterables are unpacked
             for value in content:
                 self._preprocess(value, target_list, target_attributes,
                     handle_text)
-        elif isinstance(content, collections.Callable):
+            return
+        except TypeError:
+            pass
+        try: # callables without arguments are called
             value = content()
             self._preprocess(value, target_list, target_attributes,
                 handle_text)
-        else:
-            value = _unicode(content)
-            handle_text(value, target_list)
+            return
+        except TypeError:
+            pass
+        # everything else is converted to Unicode
+        value = _unicode(content)
+        handle_text(value, target_list)
 
     def _parse_xml_fragment(self, xml_fragment):
         try:
             xml_fragment_parser = self.__dict__['_v_xml_fragment_parser']
         except KeyError:
             from ecoxipy.parsing import XMLFragmentParser
-            xml_fragment_parser = XMLFragmentParser(
-                self._output, self._parser)
+            xml_fragment_parser = XMLFragmentParser(self._output,
+                self._parser)
             self._v_xml_fragment_parser = xml_fragment_parser
-        parsed = xml_fragment_parser.parse(xml_fragment)
-        return parsed
+        return xml_fragment_parser.parse(xml_fragment)
 
     def _slice(self, key):
         if key.start is None: # Document
@@ -266,9 +300,10 @@ class MarkupBuilder(object):
             else:
                 encoding = _unicode(key.step)
             def create_document(*children):
-                new_children = []
+                new_children = self._deque()
                 for child in children:
-                    self._preprocess(child, new_children)
+                    self._preprocess(child, new_children, None,
+                        self._append_text)
                 return self._output.document(doctype_name,
                     doctype_publicid, doctype_systemid, new_children,
                     omit_xml_declaration, encoding)
@@ -285,10 +320,11 @@ class MarkupBuilder(object):
     def _item(self, key):
         key = self._prepare_text(key)
         def build(*children, **attributes):
-            new_children = []
+            new_children = self._deque()
             new_attributes = {}
             for child in children:
-                self._preprocess(child, new_children, new_attributes)
+                self._preprocess(child, new_children, new_attributes,
+                    self._append_text)
             for attr_name, value in attributes.items():
                 new_attributes[_unicode(attr_name)] = self._prepare_text(value)
             return self._output.element(key, new_children, new_attributes)
@@ -297,7 +333,7 @@ class MarkupBuilder(object):
     def __getattr__(self, name):
         '''\
         Return an
-        :ref:`element-creating function <ecoxipy.MarkupBuilder.item_access>`.
+        :ref:`element-creating function <ecoxipy.MarkupBuilder.elements>`.
 
         :param name: The element name.
         :returns: An element in the output representation.
@@ -307,7 +343,7 @@ class MarkupBuilder(object):
     def __getitem__(self, key):
         '''\
         Return an
-        :ref:`element-creating function <ecoxipy.MarkupBuilder.item_access>`
+        :ref:`element-creating function <ecoxipy.MarkupBuilder.elements>`
         for item access or either a
         :ref:`processing instruction or a document <ecoxipy.MarkupBuilder.slicing>`
         for slicing.
@@ -316,7 +352,7 @@ class MarkupBuilder(object):
         :returns: An element, a processing instruction or a document in the
             output representation.
         '''
-        if isinstance(key, slice):
+        if key.__class__ is slice:
             return self._slice(key)
         else:
             return self._item(key)
@@ -331,12 +367,10 @@ class MarkupBuilder(object):
             representation.
         :returns: Objects of the output representation.
         '''
-        processed_content = []
+        processed_content = self._deque()
         for content_element in content:
             self._preprocess(content_element, processed_content, None,
-                lambda text, target_list: target_list.extend(
-                    self._parse_xml_fragment(text))
-            )
+                self._append_xml)
         return processed_content
 
     def __and__(self, content):
@@ -368,12 +402,14 @@ class Output(object):
     @abstractmethod
     def is_native_type(self, content):
         '''\
-        Tests if an object of an type is to be decoded or converted to unicode
-        or not.
+        Tests if an object is native to the output representation. This is
+        used by :class:`MarkupBuilder` to test children of elements and
+        documents if they can be fed to the :class:`Output` implementation
+        without conversion.
 
         :param content: The object to test.
-        :returns: :const:`True` for an object not to decode or convert to
-            Unicode, :const:`False` otherwise.
+        :returns: :const:`True` for an object native to the output
+            representation, :const:`False` otherwise.
         '''
 
     @abstractmethod
@@ -382,10 +418,12 @@ class Output(object):
         Creates an element representation.
 
         :param name: The name of the element to create.
-        :param children: The list of children to add to the element to
-            create.
-        :type children: :func:`list`
+        :type name: Unicode string
+        :param children: The children to add to the element to create. This
+            contains only objects native to the output representation.
+        :type children: :class:`collections.deque`
         :param attributes: The mapping of attributes of the element to create.
+            Keys and values are Unicode strings.
         :type attributes: :class:`dict`
         :returns: The element representation created.
         '''
@@ -396,6 +434,7 @@ class Output(object):
         Creates a text node representation.
 
         :param content: The text.
+        :type content: Unicode string
         :returns: The created text representation.
         '''
 
@@ -405,8 +444,8 @@ class Output(object):
         Creates a comment representation.
 
         :param content: The content of the comment.
-        :returns:
-            The created comment representation.
+        :type content: Unicode string
+        :returns: The created comment representation.
         '''
 
     @abstractmethod
@@ -415,9 +454,10 @@ class Output(object):
         Creates a processing instruction representation.
 
         :param target: The target of the processing instruction.
+        :type target: Unicode string
         :param content: The content of the processing instruction.
-        :returns:
-            The created processing instruction representation.
+        :type content: Unicode string
+        :returns: The created processing instruction representation.
         '''
 
     @abstractmethod
@@ -427,11 +467,14 @@ class Output(object):
         Creates a XML document representation.
 
         :param doctype_name:  The document element name.
+        :type doctype_name: Unicode string
         :param doctype_publicid:  The document type system ID.
+        :type doctype_publicid: Unicode string
         :param doctype_systemid:  The document type system ID.
-        :param children: The list of children to add to the document to
-            create.
-        :type children: :func:`list`
+        :type doctype_systemid: Unicode string
+        :param children: The root nodes of the document to create. This
+            contains only objects are native to the output representation.
+        :type children: :class:`collections.deque`
         :param omit_xml_declaration: If :const:`True` the XML declaration is
             omitted.
         :type omit_xml_declaration: :func:`bool`
